@@ -6,8 +6,7 @@ import pytest
 from pythonfmu.builder import FmuBuilder
 
 pytestmark = pytest.mark.skipif(
-    not FmuBuilder.has_binary(), 
-    reason="No binary available for the current platform."
+    not FmuBuilder.has_binary(), reason="No binary available for the current platform."
 )
 pyfmi = pytest.importorskip(
     "pyfmi", reason="pyfmi is required for testing the produced FMU"
@@ -20,9 +19,7 @@ DEMO = "pythonslave.py"
 def test_integration_demo(tmp_path):
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
-
-    fmu = tmp_path / "PythonSlave.fmu"
+    fmu = FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
     assert fmu.exists()
     model = pyfmi.load_fmu(str(fmu))
     res = model.simulate(final_time=0.5)
@@ -34,9 +31,7 @@ def test_integration_demo(tmp_path):
 def test_integration_reset(tmp_path):
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
-
-    fmu = tmp_path / "PythonSlave.fmu"
+    fmu = FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
     assert fmu.exists()
 
     vr = 5  # realOut
@@ -56,13 +51,11 @@ def test_integration_reset(tmp_path):
 def test_integration_get_state(tmp_path):
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(
+    fmu = FmuBuilder.build_FMU(
         script_file,
         dest=tmp_path,
         needsExecutionTool="false",
         canGetAndSetFMUstate="true")
-
-    fmu = tmp_path / "PythonSlave.fmu"
     assert fmu.exists()
 
     model = pyfmi.load_fmu(str(fmu))
@@ -97,13 +90,11 @@ def test_integration_get_serialize_state(tmp_path):
 
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(
+    fmu = FmuBuilder.build_FMU(
         script_file,
         dest=tmp_path,
         canGetAndSetFMUstate="true",
         canSerializeFMUstate="true")
-
-    fmu = tmp_path / "PythonSlave.fmu"
     assert fmu.exists()
 
     model_description = fmpy.read_model_description(fmu)
@@ -115,7 +106,9 @@ def test_integration_get_serialize_state(tmp_path):
         modelIdentifier=model_description.coSimulation.modelIdentifier,
         instanceName='instance1')
 
-    realOut = filter(lambda var: var.name == "realOut", model_description.modelVariables)
+    realOut = filter(
+        lambda var: var.name == "realOut", model_description.modelVariables
+    )
     vrs = list(map(lambda var: var.valueReference, realOut))
     t = 0.0
     dt = 0.1
@@ -154,9 +147,7 @@ def test_integration_get_serialize_state(tmp_path):
 def test_integration_get(tmp_path):
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
-
-    fmu = tmp_path / "PythonSlave.fmu"
+    fmu = FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
     assert fmu.exists()
     model = pyfmi.load_fmu(str(fmu))
 
@@ -166,22 +157,24 @@ def test_integration_get(tmp_path):
         "realOut": 3.0,
         "booleanVariable": True,
         "stringVariable": "Hello World!",
-        "realIn": 2. / 3.,
+        "realIn": 2.0 / 3.0,
         "booleanParameter": False,
-        "stringParameter": "dog"
+        "stringParameter": "dog",
+        "container.someReal": 99.0,
+        "container.subContainer.someInteger": -15
     }
 
     variables = model.get_model_variables()
     for key, value in to_test.items():
         var = variables[key]
         if var.type == pyfmi.fmi.FMI2_INTEGER:
-            model_value = model.get_integer([var.value_reference, ])[0]
+            model_value = model.get_integer([var.value_reference])[0]
         elif var.type == pyfmi.fmi.FMI2_REAL:
-            model_value = model.get_real([var.value_reference, ])[0]
+            model_value = model.get_real([var.value_reference])[0]
         elif var.type == pyfmi.fmi.FMI2_BOOLEAN:
-            model_value = model.get_boolean([var.value_reference, ])[0]
+            model_value = model.get_boolean([var.value_reference])[0]
         elif var.type == pyfmi.fmi.FMI2_STRING:
-            model_value = model.get_string([var.value_reference, ])[0]
+            model_value = model.get_string([var.value_reference])[0]
         else:
             pytest.xfail("Unsupported type")
 
@@ -192,34 +185,34 @@ def test_integration_get(tmp_path):
 def test_integration_set(tmp_path):
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
-
-    fmu = tmp_path / "PythonSlave.fmu"
+    fmu = FmuBuilder.build_FMU(script_file, dest=tmp_path, needsExecutionTool="false")
     assert fmu.exists()
     model = pyfmi.load_fmu(str(fmu))
 
     to_test = {
         "intParam": 20,
-        "realIn": 1. / 3.,
+        "realIn": 1.0 / 3.0,
         "booleanParameter": True,
-        "stringParameter": "cat"
+        "stringParameter": "cat",
+        "container.someReal": 42.0,
+        "container.subContainer.someInteger": 421
     }
 
     variables = model.get_model_variables()
     for key, value in to_test.items():
         var = variables[key]
         if var.type == pyfmi.fmi.FMI2_INTEGER:
-            model.set_integer([var.value_reference, ], [value, ])
-            model_value = model.get_integer([var.value_reference, ])[0]
+            model.set_integer([var.value_reference], [value])
+            model_value = model.get_integer([var.value_reference])[0]
         elif var.type == pyfmi.fmi.FMI2_REAL:
-            model.set_real([var.value_reference, ], [value, ])
-            model_value = model.get_real([var.value_reference, ])[0]
+            model.set_real([var.value_reference], [value])
+            model_value = model.get_real([var.value_reference])[0]
         elif var.type == pyfmi.fmi.FMI2_BOOLEAN:
-            model.set_boolean([var.value_reference, ], [value, ])
-            model_value = model.get_boolean([var.value_reference, ])[0]
+            model.set_boolean([var.value_reference], [value])
+            model_value = model.get_boolean([var.value_reference])[0]
         elif var.type == pyfmi.fmi.FMI2_STRING:
-            model.set_string([var.value_reference, ], [value, ])
-            model_value = model.get_string([var.value_reference, ])[0]
+            model.set_string([var.value_reference], [value])
+            model_value = model.get_string([var.value_reference])[0]
         else:
             pytest.xfail("Unsupported type")
 
@@ -234,9 +227,7 @@ def test_simple_integration_fmpy(tmp_path):
 
     script_file = Path(__file__).parent / DEMO
 
-    FmuBuilder.build_FMU(script_file, dest=tmp_path)
-
-    fmu = tmp_path / "PythonSlave.fmu"
+    fmu = FmuBuilder.build_FMU(script_file, dest=tmp_path)
     assert fmu.exists()
     res = fmpy.simulate_fmu(str(fmu), stop_time=2.0)
 
@@ -249,8 +240,6 @@ def test_integration_has_local_dep(tmp_path):
 from pythonfmu.fmi2slave import Fmi2Slave, Fmi2Causality, Integer, Real, Boolean, String
 from localmodule import get_amplitude, get_time_constant
 
-slave_class = "PythonSlaveWithDep"
-
 
 class PythonSlaveWithDep(Fmi2Slave):
 
@@ -281,14 +270,12 @@ def get_time_constant():
     local_file = script_file.parent / "localmodule.py"
     local_file.write_text(local_module)
 
-    FmuBuilder.build_FMU(
+    fmu = FmuBuilder.build_FMU(
         script_file,
         dest=tmp_path,
         project_files=[local_file],
         needsExecutionTool="false",
     )
-
-    fmu = tmp_path / "PythonSlaveWithDep.fmu"
     assert fmu.exists()
     model = pyfmi.load_fmu(str(fmu))
     res = model.simulate(final_time=0.5)
@@ -296,3 +283,38 @@ def get_time_constant():
     assert res["realOut"][-1] == pytest.approx(
         22.0 * 5.0 * math.exp(res["time"][-1] / 0.1), rel=1e-7
     )
+
+
+@pytest.mark.integration
+def test_integration_throw_py_error(tmp_path):
+    fmpy = pytest.importorskip(
+        "fmpy", reason="fmpy is not available for testing the produced FMU"
+    )
+
+    slave_code = """from pythonfmu.fmi2slave import Fmi2Slave, Fmi2Causality, Integer, Real, Boolean, String
+
+
+class PythonSlaveWithException(Fmi2Slave):
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+        self.realIn = 22.0
+        self.realOut = 0.0
+        self.register_variable(Real("realIn", causality=Fmi2Causality.input))
+        self.register_variable(Real("realOut", causality=Fmi2Causality.output))
+
+    def do_step(self, current_time, step_size):
+        raise RuntimeError()
+        return True
+"""
+
+    script_file = tmp_path / "orig" / "slavewithexception.py"
+    script_file.parent.mkdir(parents=True, exist_ok=True)
+    script_file.write_text(slave_code)
+
+    fmu = FmuBuilder.build_FMU(script_file, dest=tmp_path)
+    assert fmu.exists()
+
+    with pytest.raises(Exception):
+        fmpy.simulate_fmu(str(fmu), stop_time=1.0)
